package io.tswf.transmutations.view

import groovy.transform.CompileStatic
import io.tswf.transmutations.view.annotation.FieldAccessor
import io.tswf.transmutations.view.annotation.View
import io.tswf.transmutations.view.generation.ViewInjectionContext
import io.tswf.transmutations.view.generation.ViewInjector
import io.tswf.transmutations.view.reading.ViewMetadataReader

@CompileStatic
class ViewApp {
    static void main(String[] args) {
        def viewClass = TestView.toClassNode()
        def viewMetadata = ViewMetadataReader.readViewMetadata(viewClass)

        def targetClass = TestViewInjectionTarget.toClassNode()

        ViewInjector.injectViews(new ViewInjectionContext(
                targetClass: targetClass,
                viewMeta: viewMetadata
        ))

        def classBytes = targetClass.toByteArray()
        new File("view-class.class").bytes = classBytes
    }
}

@View(targetClasses = [ TestViewInjectionTarget ])
interface TestView {
    double getDoubleField()
    void setDoubleField(double d)
    String callToString()
    String superCallToString(int i1, int i2, Object s1)
}

@CompileStatic
class TestViewInjectionTarget {

    private final double doubleField

    @Override
    public String toString() {
        return "TestViewInjectionTarget{}";
    }
}

class Class1 extends Class2{
    @Override
    void someVoid() {
        super.someVoid()
    }
}

class Class2 extends Class3{

}

class Class3 {
    void someVoid() {  }
}