package io.tswf.transmutations.view.exception

import groovy.transform.CompileStatic
import io.tswf.transmutations.view.meta.ViewMethodMetadata
import org.objectweb.asm.tree.ClassNode
import org.objectweb.asm.tree.MethodNode

@CompileStatic
class ConflictResolveException extends RuntimeException {
    ViewMethodMetadata viewMethodMeta
    ClassNode targetClass
    MethodNode conflictingMethod

    ConflictResolveException(ViewMethodMetadata viewMethodMeta, ClassNode targetClass, MethodNode conflictingMethod) {
        super(getExceptionMessage("", viewMethodMeta, targetClass, conflictingMethod))
        this.viewMethodMeta = viewMethodMeta
        this.targetClass = targetClass
        this.conflictingMethod = conflictingMethod
    }

    ConflictResolveException(String message, ViewMethodMetadata viewMethodMeta, ClassNode targetClass, MethodNode conflictingMethod) {
        super(getExceptionMessage(message, viewMethodMeta, targetClass, conflictingMethod))
        this.viewMethodMeta = viewMethodMeta
        this.targetClass = targetClass
        this.conflictingMethod = conflictingMethod
    }

    ConflictResolveException(String message, Throwable cause, ViewMethodMetadata viewMethodMeta, ClassNode targetClass, MethodNode conflictingMethod) {
        super(getExceptionMessage(message, viewMethodMeta, targetClass, conflictingMethod), cause)
        this.viewMethodMeta = viewMethodMeta
        this.targetClass = targetClass
        this.conflictingMethod = conflictingMethod
    }

    ConflictResolveException(Throwable cause, ViewMethodMetadata viewMethodMeta, ClassNode targetClass, MethodNode conflictingMethod) {
        super(getExceptionMessage("", viewMethodMeta, targetClass, conflictingMethod), cause)
        this.viewMethodMeta = viewMethodMeta
        this.targetClass = targetClass
        this.conflictingMethod = conflictingMethod
    }


    static String getExceptionMessage(String message, ViewMethodMetadata viewMethodMeta, ClassNode targetClass, MethodNode conflictingMethod) {
        return "Exception occurred while trying to handle conflict for method ${conflictingMethod.javaFullName} in ${targetClass.javaName} with view ${viewMethodMeta.view.viewClass}: ${message}"
    }
}

interface IConflictResolveExceptionBypassMarker {

}
