package io.tswf.transmutations.view.exception

import groovy.transform.CompileStatic
import io.tswf.transmutations.view.meta.ViewMethodMetadata
import io.tswf.transmutations.view.reading.IViewMethodMapperReader

@CompileStatic
class MethodMapperReadingException extends RuntimeException {
    ViewMethodMetadata viewMethodMeta
    IViewMethodMapperReader mapperReader

    MethodMapperReadingException(ViewMethodMetadata viewMethodMeta, IViewMethodMapperReader mapperReader) {
        this.viewMethodMeta = viewMethodMeta
        this.mapperReader = mapperReader
    }

    MethodMapperReadingException(String message, ViewMethodMetadata viewMethodMeta, IViewMethodMapperReader mapperReader) {
        super(getExceptionMessage(message, viewMethodMeta, mapperReader))
        this.viewMethodMeta = viewMethodMeta
        this.mapperReader = mapperReader
    }

    MethodMapperReadingException(String message, Throwable cause, ViewMethodMetadata viewMethodMeta, IViewMethodMapperReader mapperReader) {
        super(getExceptionMessage(message, viewMethodMeta, mapperReader), cause)
        this.viewMethodMeta = viewMethodMeta
        this.mapperReader = mapperReader
    }

    MethodMapperReadingException(Throwable cause, ViewMethodMetadata viewMethodMeta, IViewMethodMapperReader mapperReader) {
        super(getExceptionMessage("", viewMethodMeta, mapperReader), cause)
        this.viewMethodMeta = viewMethodMeta
        this.mapperReader = mapperReader
    }

    static String getExceptionMessage(String message, ViewMethodMetadata viewMethodMeta, IViewMethodMapperReader mapperReader) {
        return "Exception occurred while trying to read view method mapper ${viewMethodMeta.viewMethod.javaFullName} from ${viewMethodMeta.view.viewClass.javaName} by ${mapperReader}: ${message}"
    }
}
