package io.tswf.transmutations.view.reading

import groovy.transform.CompileStatic
import io.tswf.transmutations.common.remapping.TransmutationRemapper
import io.tswf.transmutations.view.annotation.View
import io.tswf.transmutations.view.exception.InvalidViewClassException
import io.tswf.transmutations.view.meta.ViewMetadata
import org.objectweb.asm.Type
import org.objectweb.asm.tree.ClassNode

import java.lang.reflect.Modifier

@CompileStatic
class ViewMetadataReader {

    public static final String VIEW_TARGET_CLASSES_PARAMETER_NAME = "targetClasses"
    public static final String VIEW_TARGET_CLASSES_NAMES_PARAMETER_NAME = "targetInternalNames"

    static ViewMetadata readViewMetadata(ClassNode viewClass) {
        // Проверяем, что поданный нам класс это интейрфейс
        if (!Modifier.isInterface(viewClass.access)) {
            throw new InvalidViewClassException("only interfaces can be views", viewClass)
        }

        // Вычитываем простые параметры из аннотации View
        def viewAnnotation = viewClass.getAnnotationByType(View)
        def result = new ViewMetadata()
        result.viewClass = viewClass

        def rawTargets = new HashSet<Type>()

        viewAnnotation.findValueByName(VIEW_TARGET_CLASSES_PARAMETER_NAME, Type[])?.tap {
            rawTargets.addAll(Arrays.asList(it))
        }

        viewAnnotation.findValueByName(VIEW_TARGET_CLASSES_NAMES_PARAMETER_NAME, String[])?.tap {
            rawTargets.addAll(map {name -> Type.getObjectType(name) })
        }

        // Убеждаемся, что у View указана хотя бы одна цель
        if (rawTargets.isEmpty()) {
            throw new InvalidViewClassException("there is no target classes chosen", viewClass)
        }

        // Читаем метадату методов интерфейса
        def methodsToRead = viewClass.methods
            .filter { Modifier.isAbstract(it.access) }

        //TODO: Может быть сделать тут пооптимальнее? 
        for (def target : rawTargets) {
            for (def methodToRead : methodsToRead) {
                def methodViewMetadata = ViewMethodMetadataReader.readViewMethodMetadata(result, methodToRead, target)
                result.viewMethodsMetadata << methodViewMetadata
            }
        }

        result.targets = rawTargets.mapToCollection({ TransmutationRemapper.mapType(it) }, { new HashSet<Type>() })

        return result
    }
}
