package io.tswf.transmutations.view.reading

import groovy.transform.CompileStatic
import io.tswf.transmutations.common.conflict.ConflictStrategyReader
import io.tswf.transmutations.common.remapping.TransmutationRemapper
import io.tswf.transmutations.view.exception.MethodMapperReadingException
import io.tswf.transmutations.view.exception.MultipleViewMethodMappersFoundException
import io.tswf.transmutations.view.exception.NoViewMethodMapperFoundException
import io.tswf.transmutations.view.meta.ViewMetadata
import io.tswf.transmutations.view.meta.ViewMethodMetadata
import org.objectweb.asm.Type
import org.objectweb.asm.tree.MethodNode

@CompileStatic
class ViewMethodMetadataReader {

    @Lazy
    static List<IViewMethodMapperReader> methodMapperReaders = ServiceLoader.load(IViewMethodMapperReader)
        .toList()

    static ViewMethodMetadata readViewMethodMetadata(ViewMetadata viewMetadata, MethodNode methodNode, Type rawTargetType) {
        def result = new ViewMethodMetadata(
                viewMethod: methodNode,
                view: viewMetadata,
                mapper: null,
                targetType: rawTargetType
        )

        result.conflictStrategy = ConflictStrategyReader.readConflictStrategy(result.viewMethod)

        def mapperReadingContext = new ViewMethodMapperReadingContext(viewMethodMeta: result)
        for (def methodMapperReader : methodMapperReaders) {
            try {
                methodMapperReader.handleViewMethodReading(mapperReadingContext)
            } catch (Throwable e) {
                throw new MethodMapperReadingException(e, result, methodMapperReader)
            }
        }

        if (mapperReadingContext.result.isNullOrEmpty()) {
            throw new NoViewMethodMapperFoundException(result)
        }

        if (mapperReadingContext.result.size() > 1) {
            throw new MultipleViewMethodMappersFoundException(result, mapperReadingContext.result)
        }

        result.mapper = mapperReadingContext.result.first()
        result.targetType = TransmutationRemapper.mapType(rawTargetType)

        return result
    }
}
