package io.tswf.transmutations.view.reading.mappers

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.transmutations.view.annotation.MethodInvoke
import io.tswf.transmutations.view.annotation.unsafe.SuperMethodInvoke
import io.tswf.transmutations.view.exception.InvalidViewMethodException
import io.tswf.transmutations.view.generation.mapper.MethodInvokeMapper
import io.tswf.transmutations.view.generation.mapper.SuperMethodInvokeMapper
import io.tswf.transmutations.view.reading.IViewMethodMapperReader
import io.tswf.transmutations.view.reading.ViewMethodMapperReadingContext
import io.tswf.transmutations.view.util.MethodToInvokeResolveHelper
import org.objectweb.asm.commons.Method

@AutoService(IViewMethodMapperReader)
@CompileStatic
class SuperMethodInvokeMapperReader extends AbstractMethodInvokeMapperReader {

    public static final Set<String> PREFIXES = MethodInvokeMapperReader.PREFIXES.map { "super${it.with { length() == 1 ? charAt(0).toUpperCase() : "${charAt(0).toUpperCase()}${substring(1)}" }}".toString() } as Set

    SuperMethodInvokeMapperReader() {
        super(SuperMethodInvoke, PREFIXES)
    }

    @Override
    void onMethodReaded(ViewMethodMapperReadingContext readingContext, Method methodDescription) {
        readingContext.result << new SuperMethodInvokeMapper(methodDescription)
    }
}
