package io.tswf.transmutations.view.util

import groovy.transform.CompileStatic
import io.tswf.transmutations.view.exception.InvalidViewMethodException
import io.tswf.transmutations.view.meta.ViewMethodMetadata

@CompileStatic
class MethodToInvokeResolveHelper {

    static String resolveTargetMethodNameFromViewMethod(ViewMethodMetadata viewMethodMeta, Collection<String> prefixes) {
        def viewMethod = viewMethodMeta.viewMethod
        def methodName = viewMethod.name

        for (def prefix : prefixes) {
            if (methodName.startsWith(prefix) && methodName.length() > prefix.length()) {
                def prunedMethodName = methodName
                        .substring(prefix.length())
                        .with {
                            length() > 1 ? "${charAt(0).toLowerCase()}${substring(1)}" : charAt(0).toLowerCase()
                        }

                return prunedMethodName
            }
        }

        throw new InvalidViewMethodException("target method name not passed in annotation and can't be extracted from view method name. Please, use someone from ${prefixes}, e.g. ${prefixes.randomElement}HelloWorldMethod", viewMethodMeta)
    }
}
