package io.tswf.transmutations.view.util

import groovy.transform.CompileStatic
import io.tswf.transmutations.view.exception.InvalidViewMethodException
import io.tswf.transmutations.view.meta.ViewMethodMetadata
import org.objectweb.asm.Type
import org.objectweb.asm.commons.Method
import org.objectweb.asm.tree.InsnList
import org.objectweb.asm.tree.TypeInsnNode
import org.objectweb.asm.tree.VarInsnNode

import static org.objectweb.asm.Opcodes.CHECKCAST
import static org.objectweb.asm.Opcodes.ILOAD

@CompileStatic
class ParametersHelper {

    static void validateMethodCallParameters(Method methodToCall, ViewMethodMetadata viewMethodMeta) {
        def viewMethodParameterTypes = viewMethodMeta.viewMethod.parameterTypes
        def methodToInvokeParameterTypes = Type.getArgumentTypes(methodToCall.descriptor)

        // Сравниваем параметры метода из View и метода, который нужно вызвать
        if (viewMethodParameterTypes.size() != methodToInvokeParameterTypes.size()) {
            throw new InvalidViewMethodException("beause target method parameters count not equals to method parameters count (${viewMethodParameterTypes.size()} in view, ${methodToInvokeParameterTypes.size()} in target)", viewMethodMeta)
        }

        for (def parameterToValidateIndex = 0; parameterToValidateIndex < viewMethodParameterTypes.size(); parameterToValidateIndex ++) {
            def viewParameterType = viewMethodParameterTypes[parameterToValidateIndex]
            def methodToInvokeParameterType = methodToInvokeParameterTypes[parameterToValidateIndex]

            if (viewParameterType != methodToInvokeParameterType && !(!methodToInvokeParameterType.isPrimitive() && viewParameterType.descriptor == Object.asmDescriptor)) {
                throw new InvalidViewMethodException("beause it ${parameterToValidateIndex}'st parameter does not matches with target method parameter. Expected ${methodToInvokeParameterType.className}, but was ${viewParameterType.className}", viewMethodMeta)
            }
        }
    }

    static void loadMethodCallParameters(InsnList insns, Method methodToCall, ViewMethodMetadata viewMethodMeta) {
        def viewMethodParameterTypes = viewMethodMeta.viewMethod.parameterTypes
        def methodToInvokeParameterTypes = Type.getArgumentTypes(methodToCall.descriptor)

        // Загружаем на стек параметры вызываемого метода
        for (def parameterToLoadIndex = 0; parameterToLoadIndex < viewMethodParameterTypes.size(); parameterToLoadIndex ++) {
            def viewParameterType = viewMethodParameterTypes[parameterToLoadIndex]
            def methodToInvokeParameterType = methodToInvokeParameterTypes[parameterToLoadIndex]

            insns << new VarInsnNode(viewParameterType.getOpcode(ILOAD), parameterToLoadIndex + 1)

            // Для не примитивных параметров на всякий случай делаем каст
            if (!methodToInvokeParameterType.isPrimitive()) {
                insns << new TypeInsnNode(CHECKCAST, methodToInvokeParameterType.internalName)
            }
        }
    }
}
